/*
	SuperCollider real time audio synthesis system
    Copyright (c) 2002 James McCartney. All rights reserved.
	http://www.audiosynth.com

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
*/


#include "SC_PlugIn.h"

static InterfaceTable *ft;

struct ___PROJECTNAMEASIDENTIFIER___ : public Unit
{
	// persistent property of this UGen. "m" prefix denotes "member'.
	float mPhase;
	float mFreqMul;
};


//////////////////////////////////////////////////////////////////////////////////////////////////


extern "C"
{
	void load(InterfaceTable *inTable);

	void ___PROJECTNAMEASIDENTIFIER____next_a(___PROJECTNAMEASIDENTIFIER___ *unit, int inNumSamples);
	void ___PROJECTNAMEASIDENTIFIER____next_k(___PROJECTNAMEASIDENTIFIER___ *unit, int inNumSamples);
	void ___PROJECTNAMEASIDENTIFIER____Ctor(___PROJECTNAMEASIDENTIFIER___ *unit);
}


//////////////////////////////////////////////////////////////////////////////////////////////////


void ___PROJECTNAMEASIDENTIFIER____next_a(___PROJECTNAMEASIDENTIFIER___ *unit, int inNumSamples)
{
	float *out = ZOUT(0);
	float *freq = ZIN(0);
	
	// storing struct params locally causes them to be loaded into a register.
	float freqmul = unit->mFreqMul;
	float phase = unit->mPhase;

	// store state of random number generator
	RGET

	// out must be written last for in-place operation (ie, ZIN(0) == ZOUT(0))
	for (int i = 0; i < inNumSamples; i++)
	{
		float z = phase;
		
		phase += freq[i] * freqmul;
		if (phase >= 1.f) phase -= 2.f;
		else if (phase <= -1.f) phase += 2.f;
		
		out[i] = z;
	}
	
	unit->mPhase = phase;

	RPUT
}


//////////////////////////////////////////////////////////////////////////////////////////////////


void ___PROJECTNAMEASIDENTIFIER____next_k(___PROJECTNAMEASIDENTIFIER___ *unit, int inNumSamples)
{
	float *out = ZOUT(0);
	float freq = ZIN0(0) * unit->mFreqMul;
	
	// store struct param locally to load it into a register.
	float phase = unit->mPhase;
	
	// store state of random number generator
	RGET
	
	if (freq > 0.f)
	{
		for (int i = 0; i < inNumSamples; i++)
		{
			out[i] = phase;
			
			phase += freq;
			if (phase >= 1.f) phase -= 2.f;
		}
	}
	else
	{
		for (int i = 0; i < inNumSamples; i++)
		{
			out[i] = phase;
			
			phase += freq;
			if (phase <= -1.f) phase += 2.f;
		}		
	}

	unit->mPhase = phase;
	
	RPUT
}


//////////////////////////////////////////////////////////////////////////////////////////////////


void ___PROJECTNAMEASIDENTIFIER____Ctor(___PROJECTNAMEASIDENTIFIER___* unit)
{
	// set the calculation function based on signal rate.
	if (INRATE(0) == calc_FullRate)
		SETCALC(___PROJECTNAMEASIDENTIFIER____next_a);
	else
		SETCALC(___PROJECTNAMEASIDENTIFIER____next_k);

	RGET

	unit->mPhase = IN0(1);
	unit->mFreqMul = 2.0 * SAMPLEDUR;
	
	RPUT

	___PROJECTNAMEASIDENTIFIER____next_k(unit, 1);
}


////////////////////////////////////////////////////////////////////////////////////////////////////////


void load(InterfaceTable *inTable)
{
	ft = inTable;

	DefineSimpleUnit(___PROJECTNAMEASIDENTIFIER___);
}


////////////////////////////////////////////////////////////////////////////////////////////////////////
